# import all relevant contents from the associated module.
from vandal.hub import toolkit
from vandal.hub import example

# coloring.
from colorama import (
    Fore,
    Back,
    Style,
    init,
)

init()

from vandal.hub.toolkit import (
    random_value,
    random_pool,
    split_values,
    join_values,
    replace_values,
    list_sort,
    index_sort,
    auto_sort,
    create_password,
    file_handler,
    save_to,
    paint_text,
)

from vandal.hub.example import linear_regression

# all relevant contents.
__all__ = [
    'random_value',
    'random_pool',
    'split_values',
    'join_values',
    'replace_values',
    'list_sort',
    'index_sort',
    'auto_sort',
    'create_password',
    'file_handler',
    'save_to',
    'paint_text',
    'toolkit',
    'example',
    'linear_regression',
]
